package com.gqy.common.utils.bean;

import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;
import java.beans.PropertyDescriptor;
import java.util.*;


/**
 * Bean 工具类
 *
 * @author raycos
 */
public class BeanUtils extends org.springframework.beans.BeanUtils
{
    /** Bean方法名中属性名开始的下标 */
    private static final int BEAN_METHOD_PROP_INDEX = 3;

    /** * 匹配getter方法的正则表达式 */
    private static final Pattern GET_PATTERN = Pattern.compile("get(\\p{javaUpperCase}\\w*)");

    /** * 匹配setter方法的正则表达式 */
    private static final Pattern SET_PATTERN = Pattern.compile("set(\\p{javaUpperCase}\\w*)");


    /**
     * Map属性填充到Bean对象
     */
    public static <T> T mapToBean(Map<String, Object> map, T bean) {
        if (map == null || bean == null) {
            return bean;
        }

        try {
            Class<?> beanClass = bean.getClass();
            Field[] fields = beanClass.getDeclaredFields();

            for (Field field : fields) {
                field.setAccessible(true);
                String fieldName = field.getName();
                Object value = map.get(fieldName);

                // 尝试下划线命名
                if (value == null) {
                    value = map.get(camelToUnderscore(fieldName));
                }

                if (value != null) {
                    Object convertedValue = convertValueToFieldType(value, field.getType());
                    if (convertedValue != null) {
                        field.set(bean, convertedValue);
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return bean;
    }

    /**
     * 转换值到指定的类型，增强的数值处理
     */
    private static Object convertValueToFieldType(Object value, Class<?> targetType) {
        if (value == null) {
            return null;
        }

        try {
            // BigDecimal类型转换增强处理
            if (targetType == BigDecimal.class) {
                return toBigDecimal(value);
            }

            // 其他基本类型转换
            if (targetType == Integer.class || targetType == int.class) {
                return toInteger(value);
            }
            if (targetType == Long.class || targetType == long.class) {
                return toLong(value);
            }
            if (targetType == Double.class || targetType == double.class) {
                return toDouble(value);
            }
            if (targetType == String.class) {
                return value.toString();
            }

            // 如果值的类型就是目标类型，直接返回
            if (targetType.isAssignableFrom(value.getClass())) {
                return value;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 增强的BigDecimal转换，处理各种可能的格式
     */
    public static BigDecimal toBigDecimal(Object value) {
        if (value == null) {
            return BigDecimal.ZERO;
        }

        if (value instanceof BigDecimal) {
            return (BigDecimal) value;
        }

        String strValue = value.toString().trim();

        // 如果为空字符串，返回0
        if (strValue.isEmpty()) {
            return BigDecimal.ZERO;
        }

        try {
            // 处理百分比格式
            if (strValue.endsWith("%")) {
                strValue = strValue.substring(0, strValue.length() - 1);
                return new BigDecimal(strValue).divide(new BigDecimal("100"));
            }

            // 处理金额格式，移除货币符号和逗号
            strValue = strValue.replaceAll("[¥$,]", "")
                    .replaceAll("\\s+", "");

            // 处理科学计数法
            if (strValue.toLowerCase().contains("e")) {
                return new BigDecimal(new BigDecimal(strValue).toPlainString());
            }

            // 处理普通数字格式
            return new BigDecimal(strValue);
        } catch (Exception e) {
            e.printStackTrace();
            return BigDecimal.ZERO;
        }
    }

    /**
     * Integer转换
     */
    private static Integer toInteger(Object value) {
        if (value == null) {
            return null;
        }
        if (value instanceof Number) {
            return ((Number) value).intValue();
        }
        try {
            String strValue = value.toString().trim();
            if (strValue.contains(".")) {
                return (int) Double.parseDouble(strValue);
            }
            return Integer.parseInt(strValue);
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * Long转换
     */
    private static Long toLong(Object value) {
        if (value == null) {
            return null;
        }
        if (value instanceof Number) {
            return ((Number) value).longValue();
        }
        try {
            String strValue = value.toString().trim();
            if (strValue.contains(".")) {
                return (long) Double.parseDouble(strValue);
            }
            return Long.parseLong(strValue);
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * Double转换
     */
    private static Double toDouble(Object value) {
        if (value == null) {
            return null;
        }
        if (value instanceof Number) {
            return ((Number) value).doubleValue();
        }
        try {
            return Double.parseDouble(value.toString().trim());
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * 驼峰转下划线
     */
    private static String camelToUnderscore(String str) {
        if (str == null || str.isEmpty()) {
            return str;
        }
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < str.length(); i++) {
            char c = str.charAt(i);
            if (Character.isUpperCase(c)) {
                if (i > 0) {
                    sb.append('_');
                }
                sb.append(Character.toLowerCase(c));
            } else {
                sb.append(c);
            }
        }
        return sb.toString();
    }


    /**
     * Bean属性复制工具方法。
     *
     * @param dest 目标对象
     * @param src 源对象
     */
    public static void copyBeanProp(Object dest, Object src)
    {
        try
        {
            // 如果源对象是Map
            if (src instanceof Map) {
                Map<String, Object> map = (Map<String, Object>) src;
                // 获取目标对象的所有属性描述符
                PropertyDescriptor[] descriptors = Introspector.getBeanInfo(dest.getClass()).getPropertyDescriptors();

                for (PropertyDescriptor descriptor : descriptors) {
                    String propertyName = descriptor.getName();
                    if (propertyName.equals("class")) {
                        continue;
                    }

                    // 获取setter方法
                    Method setter = descriptor.getWriteMethod();
                    if (setter != null && map.containsKey(propertyName)) {
                        Object value = map.get(propertyName);
                        if (value != null) {
                            // 获取setter方法的参数类型
                            Class<?> paramType = setter.getParameterTypes()[0];
                            // 转换值类型
                            Object convertedValue = convertValue(value, paramType);
                            // 设置属性值
                            setter.invoke(dest, convertedValue);
                        }
                    }
                }
            } else {
                // 原有的Bean到Bean复制逻辑
                copyProperties(src, dest);
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    /**
     * 获取对象的setter方法。
     *
     * @param obj 对象
     * @return 对象的setter方法列表
     */
    public static List<Method> getSetterMethods(Object obj)
    {
        // setter方法列表
        List<Method> setterMethods = new ArrayList<Method>();

        // 获取所有方法
        Method[] methods = obj.getClass().getMethods();

        // 查找setter方法

        for (Method method : methods)
        {
            Matcher m = SET_PATTERN.matcher(method.getName());
            if (m.matches() && (method.getParameterTypes().length == 1))
            {
                setterMethods.add(method);
            }
        }
        // 返回setter方法列表
        return setterMethods;
    }

    /**
     * 获取对象的getter方法。
     *
     * @param obj 对象
     * @return 对象的getter方法列表
     */

    public static List<Method> getGetterMethods(Object obj)
    {
        // getter方法列表
        List<Method> getterMethods = new ArrayList<Method>();
        // 获取所有方法
        Method[] methods = obj.getClass().getMethods();
        // 查找getter方法
        for (Method method : methods)
        {
            Matcher m = GET_PATTERN.matcher(method.getName());
            if (m.matches() && (method.getParameterTypes().length == 0))
            {
                getterMethods.add(method);
            }
        }
        // 返回getter方法列表
        return getterMethods;
    }

    /**
     * 检查Bean方法名中的属性名是否相等。<br>
     * 如getName()和setName()属性名一样，getName()和setAge()属性名不一样。
     *
     * @param m1 方法名1
     * @param m2 方法名2
     * @return 属性名一样返回true，否则返回false
     */

    public static boolean isMethodPropEquals(String m1, String m2)
    {
        return m1.substring(BEAN_METHOD_PROP_INDEX).equals(m2.substring(BEAN_METHOD_PROP_INDEX));
    }

    /**
     * Map转Bean
     *
     * @param map Map对象
     * @param clazz 目标Bean类型
     * @return Bean对象
     */
    public static <T> T mapToBean(Map<String, Object> map, Class<T> clazz) {
        try {
            T bean = clazz.newInstance();
            if (map != null) {
                for (Map.Entry<String, Object> entry : map.entrySet()) {
                    String propertyName = entry.getKey();
                    Object value = entry.getValue();
                    if (value != null) {
                        // 处理日期类型
                        if (value instanceof String) {
                            String strValue = (String) value;
                            if (isDateString(strValue)) {
                                value = parseDate(strValue);
                            }
                        }
                        setProperty(bean, propertyName, value);
                    }
                }
            }
            return bean;
        } catch (Exception e) {
            throw new RuntimeException("Map转Bean异常", e);
        }
    }

    /**
     * Bean转Map
     *
     * @param bean Bean对象
     * @return Map对象
     */
    public static Map<String, Object> beanToMap(Object bean) {
        if (bean == null) {
            return null;
        }
        Map<String, Object> map = new HashMap<>();
        try {
            BeanInfo beanInfo = Introspector.getBeanInfo(bean.getClass());
            PropertyDescriptor[] propertyDescriptors = beanInfo.getPropertyDescriptors();
            for (PropertyDescriptor property : propertyDescriptors) {
                String key = property.getName();
                if (key.compareToIgnoreCase("class") == 0) {
                    continue;
                }
                Method getter = property.getReadMethod();
                Object value = getter != null ? getter.invoke(bean) : null;
                map.put(key, value);
            }
            return map;
        } catch (Exception e) {
            throw new RuntimeException("Bean转Map异常", e);
        }
    }

    /**
     * 设置Bean属性值
     */
    private static void setProperty(Object bean, String propertyName, Object value) throws Exception {
        // ... existing code ...
        try {
            PropertyDescriptor property = new PropertyDescriptor(propertyName, bean.getClass());
            Method setter = property.getWriteMethod();
            if (setter != null) {
                Class<?> paramType = setter.getParameterTypes()[0];
                Object convertedValue = convertValue(value, paramType);
                setter.invoke(bean, convertedValue);
            }
        } catch (IntrospectionException e) {
            // 尝试使用另一种方式获取属性描述符
            BeanInfo beanInfo = Introspector.getBeanInfo(bean.getClass());
            PropertyDescriptor[] properties = beanInfo.getPropertyDescriptors();
            for (PropertyDescriptor pd : properties) {
                if (pd.getName().equals(propertyName)) {
                    Method setter = pd.getWriteMethod();
                    if (setter != null) {
                        Class<?> paramType = setter.getParameterTypes()[0];
                        Object convertedValue = convertValue(value, paramType);
                        setter.invoke(bean, convertedValue);
                    }
                    break;
                }
            }
        }
    }

    /**
     * 转换值类型
     */
    private static Object convertValue(Object value, Class<?> targetType) {
        if (value == null) {
            return null;
        }

        try {
            // 处理基本数据类型
            if (targetType == Integer.class || targetType == int.class) {
                return Integer.parseInt(value.toString());
            }
            if (targetType == Long.class || targetType == long.class) {
                return Long.parseLong(value.toString());
            }
            if (targetType == Double.class || targetType == double.class) {
                return Double.parseDouble(value.toString());
            }
            if (targetType == Float.class || targetType == float.class) {
                return Float.parseFloat(value.toString());
            }
            if (targetType == Boolean.class || targetType == boolean.class) {
                return Boolean.parseBoolean(value.toString());
            }
            if (targetType == Date.class && value instanceof String) {
                return parseDate((String) value);
            }
            // 如果目标类型就是String，直接返回toString结果
            if (targetType == String.class) {
                return value.toString();
            }
        } catch (Exception e) {
            throw new RuntimeException("类型转换异常: " + value + " to " + targetType, e);
        }
        return value;
    }

    /**
     * 判断是否为日期字符串
     */
    private static boolean isDateString(String str) {
        return str.matches("\\d{4}-\\d{2}-\\d{2}") ||
                str.matches("\\d{4}-\\d{2}-\\d{2} \\d{2}:\\d{2}:\\d{2}");
    }

    /**
     * 解析日期字符串
     */
    private static Date parseDate(String dateStr) {
        try {
            if (dateStr.length() == 10) {
                return new SimpleDateFormat("yyyy-MM-dd").parse(dateStr);
            }
            return new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").parse(dateStr);
        } catch (Exception e) {
            throw new RuntimeException("日期解析异常: " + dateStr, e);
        }
    }

    /**
     * List<Map>转List<Bean>
     */
    public static <T> List<T> mapListToBeanList(List<Map<String, Object>> mapList, Class<T> clazz) {
        if (mapList == null || mapList.isEmpty()) {
            return new ArrayList<>();
        }
        return mapList.stream()
                .map(map -> mapToBean(map, clazz))
                .collect(Collectors.toList());
    }

    /**
     * List<Bean>转List<Map>
     */
    public static <T> List<Map<String, Object>> beanListToMapList(List<T> beanList) {
        if (beanList == null || beanList.isEmpty()) {
            return new ArrayList<>();
        }
        return beanList.stream()
                .map(BeanUtils::beanToMap)
                .collect(Collectors.toList());
    }

}
